#include <stdlib.h>

#include "main.h"
#include "SDL/SDL.h"
#include "hypot.h"
#include "enemy.h"
#include "player.h"

#define BUL_NUM 256
#define CHANGE_FRAME_NUM 25*5

static int change=CHANGE_FRAME_NUM;

struct bullet {
	struct bullet *next,*prev;
	int dx, dy;
	int x, y;
} bul[BUL_NUM];

static struct bullet *reserve_top; /* $B@8$-$F$J$$$N(B */
static struct bullet *living_top; /* $B@8$-$F$k$N(B */

static SDL_Surface *bullet;

#define SPEED (4<<FIX_SHIFT)
#define X 150
#define Y 150

void enemy_init()
{
	int i;
	SDL_Surface *temp = SDL_LoadBMP("bul.bmp");

	bullet = SDL_ConvertSurface( temp, screen->format, SDL_SRCCOLORKEY|SDL_SWSURFACE );
	SDL_FreeSurface ( temp );

	SDL_SetColorKey( bullet, SDL_SRCCOLORKEY, green );

	for ( i=0; i<BUL_NUM-1; i++ ) {
		bul[i].next = &(bul[i+1]);
		bul[i].prev = NULL;
	}
	bul[BUL_NUM-1].next = NULL;
	reserve_top = bul;
	living_top = NULL;
}

void enemy_draw()
{
	struct bullet *p = living_top;

	while ( p ) {
		SDL_Rect dst = { p->x>>FIX_SHIFT, p->y>>FIX_SHIFT, 0, 0 };
		SDL_BlitSurface( bullet, NULL, screen, &dst );

		p = p->next;
	}
}

enum {
	BUL_NORMAL,
	BUL_SCATTER,
};

static int bul_mode= BUL_SCATTER;

static inline void add_bul(int dx, int dy)
{
	struct bullet *new_ = reserve_top;
	if ( new_ == NULL )
		return;
	reserve_top = new_->next;

	if ( living_top ) {
		living_top->prev = new_;
		new_->next = living_top;
	} else {
		new_->next = NULL;
	}
	living_top = new_;
	new_->prev = NULL;	
	
	new_->x = (X<<FIX_SHIFT);
	new_->y = (Y<<FIX_SHIFT);

	new_->dx = dx;
	new_->dy = dy;
}

#define SCATTER_NUM 30
#define SCATTER_RATE (2<<FIX_SHIFT)

static inline void new_scatter( int dx, int dy )
{
	int i;
	
	for ( i=0; i<SCATTER_NUM; i++ ) {
		add_bul( dx+rand()%SCATTER_RATE-SCATTER_RATE/2,
			 dy+rand()%SCATTER_RATE-SCATTER_RATE/2 );
	}
}

static inline void new_bul()
{
	int dx,dy,len;
	
#ifdef AIOHJOH			
	dx = ((player.x+16-X)&~0x3f);
	dy = ((player.y+16-Y)&~0x3f);
#else			
	dx = (player.x-X)+8;
	dy = (player.y-Y)+8;
#endif			
	
	len = hypot( dx, dy );
	
	if ( len != 0 ) {
		dx = (dx<<FIX_SHIFT)*SPEED/len;
		dy = (dy<<FIX_SHIFT)*SPEED/len;
	} else {
		dx = SPEED;
		dy = 0;
	}

	switch ( bul_mode ) {
	case BUL_NORMAL:
		add_bul( dx, dy );
		return;

	case BUL_SCATTER:
		new_scatter( dx, dy );
	}
}	

#define BUL_WAIT 3
static int bul_wait;

void enemy_move()
{
	struct bullet *p;

	if ( bul_wait-- <= 0 ) {
		new_bul();
		if ( bul_mode == BUL_SCATTER )
			bul_wait = BUL_WAIT*20;
		else
			bul_wait = BUL_WAIT;
	}

	if ( change-- <= 0 ) {
		bul_mode = bul_mode? BUL_NORMAL: BUL_SCATTER;
		change = CHANGE_FRAME_NUM;
	}

	p=living_top;

	while ( p ) {
		p->x += p->dx;
		p->y += p->dy;

		if ( p->x < 0-(16<<FIX_SHIFT) || p->x > (SCR_WID<<FIX_SHIFT) ||
		     p->y < 0-(16<<FIX_SHIFT) || p->y > (SCR_HGT<<FIX_SHIFT) ) {
			struct bullet *tmp = p->next;

			if ( p->prev )
				p->prev->next = p->next;
			else	/* $B@hF,(B */
				living_top = p->next;
			if ( p->next )
				p->next->prev = p->prev;
			p->next = reserve_top;
			reserve_top = p;
			p = tmp;
		} else {
			p = p->next;
		}
	}
}

void enemy_destroy()
{
	SDL_FreeSurface( bullet );
}
